import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { fromJS } from 'immutable'
import { Form, Row, Col, Input, Select, Radio, Tooltip, InputNumber, Cascader, Popover, message, AutoComplete } from 'antd'
import { QuestionCircleOutlined } from '@ant-design/icons'

import { formRule } from '@/utils/option.js'
import asyncComponent from '@/utils/asyncComponent'
import './index.scss'

const { TextArea } = Input
const ColorSketch = asyncComponent(() => import('@/mob/colorsketch'))
const CodeMirror = asyncComponent(() => import('@/templates/zshare/codemirror'))
const SourceComponent = asyncComponent(() => import('@/menu/components/share/sourcecomponent'))
const MKTable = asyncComponent(() => import('@/components/normalform/modalform/mkTable'))
const MkEditIcon = asyncComponent(() => import('@/components/mkIcon'))

const cardTypeOptions = {
  sequence: ['eleType', 'width', 'prefix', 'postfix'],
  text: ['eleType', 'datatype', 'format', 'width', 'height', 'prefix', 'postfix', 'link', 'anchors', 'noValue', 'bgImage', 'fixStyle', 'copyable', 'alignItems', 'sortField'],
  number: ['eleType', 'datatype', 'width', 'height', 'prefix', 'postfix', 'noValue', 'fixStyle', 'alignItems'],
  picture: ['eleType', 'datatype', 'width', 'lenWidRadio', 'maxWidth', 'link', 'noValue'],
  video: ['eleType', 'datatype', 'width', 'aspectRatio', 'autoPlay', 'loop', 'startTime', 'noValue', 'posterType'],
  icon: ['eleType', 'datatype', 'tipType', 'width', 'tooltip'],
  slider: ['eleType', 'datatype', 'width', 'color', 'maxValue', 'showInfo', 'showType', 'strokeWidth', 'strokeLinecap', 'trailColor'],
  splitline: ['eleType', 'color', 'width', 'borderWidth'],
  barcode: ['eleType', 'datatype', 'width', 'barHeight', 'displayValue', 'interval', 'noValue'],
  qrcode: ['eleType', 'datatype', 'width', 'qrWidth', 'color', 'url', 'noValue'],
  currentDate: ['eleType', 'width', 'dateFormat', 'prefix', 'postfix', 'fixStyle'],
  formula: ['eleType', 'width', 'height', 'eval', 'formula', 'noValue'],
  color: ['eleType', 'datatype', 'width', 'lenWidRadio', 'noValue', 'copyable'],
  tag: ['eleType', 'datatype', 'width', 'noValue', 'signs'],
}

class ElementEditForm extends Component {
  static propTpyes = {
    config: PropTypes.object,    // 组件信息
    formlist: PropTypes.any,     // 表单信息
    card: PropTypes.any,         // 按钮信息
    inputSubmit: PropTypes.any   // 回车提交事件
  }

  state = {
    formlist: null  // 表单信息
  }

  record = null

  UNSAFE_componentWillMount () {
    const { card, config, side, formlist } = this.props
    
    this.record = {}

    formlist.forEach(item => {
      this.record[item.key] = item.initVal
    })
    
    let _options = this.getOptions()

    this.setState({
      formlist: formlist.map(item => {
        item.hidden = !_options.includes(item.key)

        if ((item.key === 'field' || item.key === 'linkurl' || item.key === 'bgImage' || item.key === 'posterField') && item.type === 'select') {
          item.options = []
          
          if (side === 'sub') {
            config.subColumns.forEach(col => {
              let label = `${col.field}（${col.label}）`
              if (/^(Int|Decimal)/ig.test(col.datatype) && (card.eleType === 'number' || card.eleType === 'slider')) {
                item.options.push({
                  value: col.field,
                  text: label
                })
              } else if (/^(Nvarchar|date)/ig.test(col.datatype) && card.eleType !== 'number' && card.eleType !== 'slider') {
                item.options.push({
                  value: col.field,
                  text: label
                })
              }
            })
          } else {
            config.columns.forEach(col => {
              let label = `${col.field}（${col.label}）`
              if (/^(Int|Decimal)/ig.test(col.datatype) && (card.eleType === 'number' || card.eleType === 'slider')) {
                item.options.push({
                  value: col.field,
                  text: label
                })
              } else if (/^(Nvarchar|date)/ig.test(col.datatype) && card.eleType !== 'number' && card.eleType !== 'slider') {
                item.options.push({
                  value: col.field,
                  text: label
                })
              }
            })
          }
        } else if (item.key === 'sortField' && !item.forbid) {
          item.options = []

          config.columns.forEach(col => {
            let label = `${col.field}（${col.label}）`
            item.options.push({
              value: col.field,
              text: label
            })
          })
          
          if (config.subColumns) {
            config.subColumns.forEach(col => {
              let label = `${col.field}（${col.label}）`
              item.options.push({
                value: col.field,
                text: label
              })
            })
          }
        } else if (item.key === 'formula') {
          item.fields = []

          if (side === 'sub') {
            if (config.subColumns) {
              config.subColumns.forEach(col => {
                item.fields.push(col.field)
              })
            }
          } else {
            config.columns.forEach(col => {
              item.fields.push(col.field)
            })
          }

          item.fields = item.fields.join(', ')
        } else if (item.key === 'value') {
          item.tooltip = ''
          if (card.eleType === 'slider') {
            item.type = 'number'
            item.label = '值'
          } else if (card.eleType === 'text') {
            item.type = 'textarea'
            item.label = '内容'
            item.tooltip = '文本类型，会替换内容中的@username@、@fullName@、@mk_city@、@appname@、@bid@、@month@、@week@、@day@'
          } else if (card.eleType === 'tag') {
            item.type = 'textarea'
            item.label = '内容'
            item.tooltip = '多个值请使用逗号（英文）分隔。'
          } else {
            item.type = 'text'
            item.label = '内容'
          }
        } else if (item.key === 'format') {
          if (card.eleType === 'text') {
            item.options = item.oriOptions.filter(op => !['percent', 'thdSeparator', 'abs'].includes(op.value))
          } else if (card.eleType === 'number') {
            item.options = item.oriOptions.filter(op => !op.value || ['percent', 'thdSeparator', 'abs'].includes(op.value))
          }
        } else if (item.key === 'url') {
          item.required = card.eleType !== 'qrcode'
        }
        if (item.key === 'linkurl') {
          item.type = card.link === 'dynamic' ? item.defType : 'textarea'
        }

        return item
      })
    })
  }

  getOptions = () => {
    let _options = fromJS(cardTypeOptions[this.record.eleType]).toJS() // 选项列表
    
    if (['text', 'number', 'picture', 'slider', 'barcode', 'qrcode', 'video', 'color', 'tag'].includes(this.record.eleType)) {
      if (this.record.datatype === 'dynamic') {
        _options.push('field')
        if (this.record.eleType === 'number') {
          _options.push('decimal', 'format')
        }
      } else if (this.record.eleType === 'picture' || this.record.eleType === 'video') {
        _options.push('url')
      } else {
        _options.push('value')
      }
      
      if (this.record.eleType === 'picture' && this.record.noValue === 'show') {
        _options.push('lostTip')
      } else if (this.record.eleType === 'video' && this.record.posterType) {
        if (this.record.posterType === 'dynamic') {
          _options.push('posterField')
        } else {
          _options.push('posterUrl')
        }
      }

      if (['text', 'picture'].includes(this.record.eleType) && this.record.link) {
        _options.push('linkType')
        if (this.record.linkType === 'linkmenu') {
          _options.push('open')
          if (this.record.link === 'static') {
            _options.push('linkmenu')
          } else {
            _options.push('linkurl')
          }
        } else if (this.record.linkType === 'other') {
          _options.push('linkurl', 'open')
        } else {
          _options.push('linkurl')
        }
      } else if (this.record.eleType === 'picture' && !this.record.link) {
        _options.push('scale')
      } else if (this.record.eleType === 'slider') {
        if (this.record.showInfo === 'true') {
          _options.push('infoColor')
        }
        if (this.record.showType !== 'line') {
          _options.push('outlineWidth', 'textAlign')
        }
      }
    } else if (this.record.eleType === 'icon') {
      if (this.record.datatype === 'dynamic') {
        _options.push('field', 'noValue')
      } else if (this.record.tipType === 'text') {
        _options.push('value')
      } else {
        _options.push('icon')
      }
      if (this.record.tipType === 'text') {
        _options.push('height')
      }
    } else if (this.record.eleType === 'formula') {
      if (this.record.eval !== 'func') {
        _options.push('prefix', 'postfix', 'fixStyle', 'alignItems')
      }
      if (this.record.eval === 'true') {
        _options.push('decimal')
      }
    }
    if (_options.includes('fixStyle') && this.record.fixStyle === 'alone') {
      _options.push('fixSize', 'fixColor', 'fixLeft', 'fixRight')
    }

    return _options
  }

  /**
   * @description 下拉切换
   * 1、打开方式切换，重置可见表单和表单值
   * 2、显示位置切换，重置选择行
   * 3、切换标签类型，重置可选标签
   */
  selectChange = (key, value, option) => {
    const { config, side } = this.props

    this.record[key] = value

    if (key === 'eleType') {
      this.record.link = ''
      let _options = this.getOptions()

      if (value === 'splitline') {
        this.record.color = '#EBE9E9'
      } else if (value === 'formula') {
        this.record.decimal = ''
      }
      
      let _formlist = this.state.formlist.map(item => {
        item.initVal = this.record[item.key]
        item.hidden = !_options.includes(item.key)

        if (item.key === 'field' && item.type === 'select') {
          item.options = []

          if (side === 'sub') {
            config.subColumns.forEach(col => {
              let label = `${col.field}（${col.label}）`
              if (/^(Int|Decimal)/ig.test(col.datatype) && (value === 'number' || value === 'slider')) {
                item.options.push({
                  value: col.field,
                  text: label
                })
              } else if (/^(Nvarchar|date)/ig.test(col.datatype) && value !== 'number' && value !== 'slider') {
                item.options.push({
                  value: col.field,
                  text: label
                })
              }
            })
          } else {
            config.columns.forEach(col => {
              let label = `${col.field}（${col.label}）`
  
              if (/^(Int|Decimal)/ig.test(col.datatype) && (value === 'number' || value === 'slider')) {
                item.options.push({
                  value: col.field,
                  text: label
                })
              } else if (/^(Nvarchar|date)/ig.test(col.datatype) && value !== 'number' && value !== 'slider') {
                item.options.push({
                  value: col.field,
                  text: label
                })
              }
            })
          }
        } else if (item.key === 'value') {
          item.tooltip = ''
          if (value === 'slider') {
            item.type = 'number'
            item.label = '值'
          } else if (value === 'text') {
            item.type = 'textarea'
            item.label = '内容'
            item.tooltip = '文本类型，会替换内容中的@username@、@fullName@、@mk_city@、@appname@、@bid@、@month@、@week@、@day@'
          } else if (value === 'tag') {
            item.type = 'textarea'
            item.label = '内容'
            item.tooltip = '多个值请使用逗号（英文）分隔。'
          } else {
            item.type = 'text'
            item.label = '内容'
          }
        } else if (item.key === 'format') {
          if (value === 'text') {
            item.options = item.oriOptions.filter(op => !['percent', 'thdSeparator', 'abs'].includes(op.value))
          } else if (value === 'number') {
            item.options = item.oriOptions.filter(op => !op.value || ['percent', 'thdSeparator', 'abs'].includes(op.value))
          }
        } else if (item.key === 'url') {
          item.required = value !== 'qrcode'
        }

        return item
      })

      let _field = ''
      if (value === 'formula') {
        _field = this.props.form.getFieldValue('field') || ''
      }

      this.setState({
        formlist: _formlist
      }, () => {
        if (value === 'splitline') {
          this.props.form.setFieldsValue({width: 24, color: '#EBE9E9'})
        } else if (value === 'slider') {
          this.props.form.setFieldsValue({width: 24, color: '#1890ff'})
        } else if (value === 'qrcode') {
          this.props.form.setFieldsValue({color: '#000000'})
        } else if (value === 'text' || value === 'number') {
          this.props.form.setFieldsValue({format: ''})
        } else if (value === 'formula' && _field) {
          this.props.form.setFieldsValue({formula: '@' + _field + '@'})
        }
        if (value === 'text' || value === 'picture') {
          this.props.form.setFieldsValue({link: ''})
        }
      })
    } else if (key === 'field') {
      if (this.props.form.getFieldValue('value') !== undefined) {
        this.props.form.setFieldsValue({value: option.props.title})
      }
    } else if (key === 'link') {
      let _options = this.getOptions()
      this.setState({
        formlist: this.state.formlist.map(item => {
          item.initVal = this.record[item.key]
          item.hidden = !_options.includes(item.key)

          if (item.key === 'linkurl') {
            item.type = value === 'dynamic' ? item.defType : 'textarea'
          }
          return item
        })
      })
    } else if (['datatype', 'showInfo', 'showType', 'fixStyle', 'posterType', 'eval', 'linkType', 'tipType', 'noValue'].includes(key)) {
      let _options = this.getOptions()

      this.setState({
        formlist: this.state.formlist.map(item => {
          item.initVal = this.record[item.key]
          item.hidden = !_options.includes(item.key)

          return item
        })
      })
    }
  }

  handleSubmit = (e) => {
    e.preventDefault()

    if (this.props.inputSubmit) {
      this.props.inputSubmit()
    }
  }

  getFields() {
    const { getFieldDecorator } = this.props.form
    const fields = []

    this.state.formlist.forEach((item, index) => {
      if (item.hidden || item.forbid) return

      let label = item.label
      if (item.tooltip) {
        if (item.toolWidth) {
          label = <Tooltip placement="topLeft" overlayStyle={{maxWidth: item.toolWidth}} title={<div onClick={(e) => e.stopPropagation()}>{item.tooltip}</div>}><QuestionCircleOutlined className="mk-form-tip" />{item.label}</Tooltip>
        } else {
          label = <Tooltip placement="topLeft" title={<div onClick={(e) => e.stopPropagation()}>{item.tooltip}</div>}><QuestionCircleOutlined className="mk-form-tip" />{item.label}</Tooltip>
        }
      }

      if (item.type === 'text') {
        let rules = item.rules || []
        if (item.options && item.options.length > 0) {
          fields.push(
            <Col span={12} key={index}>
              <Form.Item label={label}>
                {getFieldDecorator(item.key, {
                  initialValue: item.initVal || '',
                  rules: [
                    {
                      required: !!item.required,
                      message: '请输入' + item.label + '!'
                    },
                    {
                      pattern: formRule.field.pattern,
                      message: formRule.field.message
                    },
                    {
                      max: formRule.input.max,
                      message: formRule.input.message
                    }
                  ]
                })(
                  <AutoComplete
                    dataSource={item.options.map((cell, i) => <AutoComplete.Option value={cell.value} key={i}>
                      {cell.value}
                    </AutoComplete.Option>)}
                    filterOption={(input, option) => option.props.children.indexOf(input) > -1}
                  >
                    <Input autoComplete="off" onPressEnter={this.handleSubmit} />
                  </AutoComplete>
                )}
              </Form.Item>
            </Col>
          )
        } else {
          fields.push(
            <Col span={12} key={index}>
              <Form.Item label={label}>
                {getFieldDecorator(item.key, {
                  initialValue: item.initVal || '',
                  rules: [
                    {
                      required: item.readonly ? false : !!item.required,
                      message: '请输入' + item.label + '!'
                    },
                    {
                      max: formRule.input.max,
                      message: formRule.input.message
                    },
                    ...rules
                  ]
                })(<Input placeholder="" autoComplete="off" disabled={item.readonly} onPressEnter={this.handleSubmit} />)}
              </Form.Item>
            </Col>
          )
        }
      } else if (item.type === 'textarea') {
        fields.push(
          <Col span={24} className="textarea" key={index}>
            <Form.Item label={label}>
              {getFieldDecorator(item.key, {
                initialValue: item.initVal || '',
                rules: [
                  {
                    required: item.readonly ? false : !!item.required,
                    message: '请输入' + item.label + '!'
                  }
                ]
              })(<TextArea autoSize={{minRows: 2}} disabled={item.readonly} placeholder={item.placeholder || ''} />)}
            </Form.Item>
          </Col>
        )
      } else if (item.type === 'number') {
        if (item.help) {
          fields.push(
            <Col span={12} key={index}>
              <Form.Item help={item.help} label={label}>
                {getFieldDecorator(item.key, {
                  initialValue: item.initVal,
                  rules: [{
                    required: item.readonly ? false : !!item.required,
                    message: '请输入' + item.label + '!'
                  }]
                })(<InputNumber min={item.min || 0} max={item.max || 10000} precision={item.precision || 0} onPressEnter={this.handleSubmit} />)}
              </Form.Item>
            </Col>
          )
        } else {
          fields.push(
            <Col span={12} key={index}>
              <Form.Item label={label}>
                {getFieldDecorator(item.key, {
                  initialValue: item.initVal,
                  rules: [{
                    required: item.readonly ? false : !!item.required,
                    message: '请输入' + item.label + '!'
                  }]
                })(<InputNumber min={item.min || 0} max={item.max || 10000} precision={item.precision || 0} onPressEnter={this.handleSubmit} />)}
              </Form.Item>
            </Col>
          )
        }
      } else if (item.type === 'select') { // 下拉搜索
        fields.push(
          <Col span={12} key={index}>
            <Form.Item label={label}>
              {getFieldDecorator(item.key, {
                initialValue: item.initVal || '',
                rules: [{
                  required: !!item.required,
                  message: '请选择' + item.label + '!'
                }]
              })(
                <Select
                  showSearch
                  allowClear={item.allowClear || false}
                  filterOption={(input, option) => option.props.children.toLowerCase().indexOf(input.toLowerCase()) >= 0}
                  onChange={(value, option) => {this.selectChange(item.key, value, option)}}
                  getPopupContainer={() => document.getElementById('card-winter')}
                >
                  {item.options.map((option, index) =>
                    <Select.Option id={`${index}`} title={option.text || option.label} key={`${index}`} value={option.value}>
                      {option.text || option.label}
                    </Select.Option>
                  )}
                </Select>
              )}
            </Form.Item>
          </Col>
        )
      } else if (item.type === 'icon') {
        fields.push(
          <Col span={12} key={index}>
            <Form.Item label={label}>
              {getFieldDecorator(item.key, {
                initialValue: item.initVal || '',
                rules: [{
                  required: !!item.required,
                  message: '请选择' + item.label + '!'
                }]
              })(
                <MkEditIcon />
              )}
            </Form.Item>
          </Col>
        )
      } else if (item.type === 'radio') {
        fields.push(
          <Col span={12} key={index}>
            <Form.Item label={label}>
              {getFieldDecorator(item.key, {
                initialValue: item.initVal,
                rules: [{
                  required: !!item.required,
                  message: '请选择' + item.label + '!'
                }]
              })(
                <Radio.Group onChange={(e) => {this.selectChange(item.key, e.target.value)}} disabled={item.readonly}>
                  {item.options.map(option => {
                    return (
                      <Radio key={option.value} value={option.value}>{option.text}</Radio>
                    )
                  })}
                </Radio.Group>
              )}
            </Form.Item>
          </Col>
        )
      } else if (item.type === 'color') {
        fields.push(
          <Col span={12} key={index} className="color-form">
            <Form.Item label={label}>
              {getFieldDecorator(item.key, {
                initialValue: item.initVal,
                rules: [
                  {
                    required: !!item.required,
                    message: '请选择' + item.label + '!'
                  }
                ]
              })(
                <ColorSketch />
              )}
            </Form.Item>
          </Col>
        )
      } else if (item.type === 'file') {
        let type = this.record.eleType
        if (item.key === 'posterUrl') {
          type = 'picture'
        }

        fields.push(
          <Col span={12} key={index}>
            <Form.Item label={label}>
              {getFieldDecorator(item.key, {
                initialValue: item.initVal,
                rules: [
                  {
                    required: !!item.required,
                    message: '请选择' + item.label + '!'
                  }
                ]
              })(
                <SourceComponent type={type} />
              )}
            </Form.Item>
          </Col>
        )
      } else if (item.type === 'cascader') {
        fields.push(
          <Col span={12} key={index}>
            <Form.Item label={label}>
              {getFieldDecorator(item.key, {
                initialValue: item.initVal || [],
                rules: [
                  {
                    required: !!item.required,
                    message: '请选择' + item.label + '!'
                  }
                ]
              })(
                <Cascader options={item.options || []} expandTrigger="hover" placeholder=""/>
              )}
            </Form.Item>
          </Col>
        )
      } else if (item.type === 'table') {
        fields.push(
          <Col span={24} key={index} className="textarea">
            <Form.Item label={label}>
              {getFieldDecorator(item.key, {
                initialValue: item.initVal || '',
                rules: [
                  {
                    required: !!item.required,
                    message: '请输入' + item.label + '!'
                  }
                ]
              })(
                <MKTable columns={item.columns || []} actions={item.actions}/>
              )}
            </Form.Item>
          </Col>
        )
      } else if (item.type === 'codemirror') {
        fields.push(
          <Col span={24} key={index} className="textarea">
            <Form.Item label={label}>
              {getFieldDecorator(item.key, {
                initialValue: item.initVal || '',
                rules: [
                  {
                    required: !!item.required,
                    message: '请输入' + item.label + '!'
                  }
                ]
              })(
                <CodeMirror mode="text/javascript"/>
              )}
            </Form.Item>
            <Popover overlayClassName="formula-fields" placement="topLeft" title="" content={<div>{item.fields}</div>} trigger="click">
              <span className="formula-icon">字段集</span>
            </Popover>
          </Col>
        )
      }
    })
    return fields
  }

  handleConfirm = () => {
    const { config } = this.props

    // 表单提交时检查输入值是否正确
    return new Promise((resolve, reject) => {
      this.props.form.validateFieldsAndScroll((err, values) => {
        if (!err) {
          values.uuid = this.props.card.uuid
          values.marks = this.props.card.marks || null

          // eslint-disable-next-line
          if (values.eleType === 'formula' && values.eval === 'true' && /^[\u4E00-\u9FA50-9a-zA-Z_\s@\+\-\*\/]*$/ig.test(values.formula) && /[\+\-\*\/]/ig.test(values.formula)) {
            let cols = []
            config.subColumns && config.subColumns.forEach(col => {
              if (/^(Int|Decimal)/ig.test(col.datatype)) {
                cols.push({reg: new RegExp('@' + col.field + '@', 'ig'), value: `(@${col.field}@)`})
              }
            })
            config.columns.forEach(col => {
              if (/^(Int|Decimal)/ig.test(col.datatype)) {
                cols.push({reg: new RegExp('@' + col.field + '@', 'ig'), value: `(@${col.field}@)`})
              }
            })

            cols.forEach(col => {
              values.formula = values.formula.replace(col.reg, col.value)
            })
          }

          if (values.width === 0) {
            let utype = {picture: '图片', video: '视频', slider: '进度条', splitline: '分割线', barcode: '条形码', qrcode: '二维码'}
            
            if (utype[values.eleType]) {
              message.warning(utype[values.eleType] + '元素宽度不可为0！')
              return
            }
          }
          
          if (values.eleType === 'tag' && values.signs && values.signs.length === 0) {
            values.signs = null
          }

          resolve(values)
        } else {
          reject(err)
        }
      })
    })
  }

  render() {
    const formItemLayout = {
      labelCol: {
        xs: { span: 24 },
        sm: { span: 7 }
      },
      wrapperCol: {
        xs: { span: 24 },
        sm: { span: 17 }
      }
    }
    return (
      <Form {...formItemLayout} className="menu-card-detail-form" id="card-winter">
        <Row gutter={24}>{this.getFields()}</Row>
      </Form>
    )
  }
}

export default Form.create()(ElementEditForm)